/*******************************************************************
     Program: TINIFILE
      Author: Matt Hamilton  (Corrections to ':ReadString' by Terry Wolfe)
                             ( Down loaded from TheOasis )
   Copyright: Released to Public domain by Original Author Matt Hamilton
                Corrections to ':ReadString' released to public domain
                by Terry Wolfe of Service Education, Inc.
        Date: 11/22/99
     Purpose: A Class to handle the .INI files
*********************************************************************/
/*--------------------------------------------
 TIniFile for XBase++

Encapsulation of the GetPrivateProfile* and WritePrivateProfile*
 functions in the Windows API.

----------------------------------------------
The methods included are:

init(AFilename)
	Creates a new TIniFile with the given filename. If the full path is
	not given, then the file is created/found in the windows directory.

  Example:  IniFile := TIniFile():New("\court32\court.ini")

ReadSections(AList)
	Returns in AList an array of all the section names in the inifile.
	AList must have already been initialised to {}.

	Example:	IniFile:ReadSections(SectionList)

ReadSection(ASection, AList)
	Returns in AList an array of all the key names in the given section.
	AList must have already been initialised to {}.

	Example:	IniFile:ReadSections("Desktop", KeyList)

ReadSectionValues(ASection, AList)
	Returns in AList an array of all the key names and their values in
	the given section.
	Each element in the resulting list will have the form "key=value".
	AList must have already been initialised to {}.

	Example:	IniFile:ReadSectionValues("Desktop", KeyList)

ReadString(ASection, AKey, ADefault)
	Returns the string specified in AKey and ASection. If the key or
	section are not found, returns ADefault. ADefault _must_ be specified.

	Example:	LastUser := IniFile:ReadString("Options", "LastUser", "Mab")

ReadInteger(ASection, AKey, ADefault)
	Returns the integer specified in AKey and ASection. If the key or
	section are not found, returns ADefault. ADefault _must_ be specified.

	Example:	Width := IniFile:ReadInteger("Options", "Width", 80)

ReadBool(ASection, AKey, ADefault)
	Returns the logical value specified in AKey and ASection. If the key or
	section are not found, returns ADefault. ADefault _must_ be specified.

	Example:	IsNew := IniFile:ReadBool("Options", "New", .t.)

WriteString(ASection, AKey, AString)
WriteInteger(ASection, AKey, AnInteger)
WriteBool(ASection, AKey, ABool)
	Writes the given data back to the ini file.

	Example:	IniFile:WriteString("Options", "LastUser", "Mab")

DeleteKey(ASection, AKey)
	Removes the given key from the ini file.

	Example:	IniFile:DeleteKey("Options", "LastUser")

EraseSection(ASection)
	Removes the given section from the ini file.

	Example:	IniFile:EraseSection("Options")

---------------------------------------------*/
#include "dll.ch"

class TIniFile
exported:
   var FileName

   method init

   method ReadSections, ReadSection, ReadSectionValues
   method ReadString, ReadInteger, ReadBool
   method WriteString, WriteInteger, WriteBool
   method DeleteKey, EraseSection
endclass

method TIniFile:init(AFileName)
   ::FileName := AFileName
return Self

method TIniFile:ReadSections(ASections)
   local n, Buffer

   Buffer := space(16383)

   GetPrivateProfileSectionNamesA(@Buffer, 16383, ::FileName)

   asize(ASections, 0)
   while( asc(Buffer) <> 0 .and. (n := at(chr(0), Buffer)) > 0 )
      aadd(ASections, left(Buffer, n - 1))
      Buffer := substr(Buffer, n + 1)
   end
return Self

method TIniFile:ReadSection(ASection, AKeys)
   local n, Buffer

   Buffer := space(16383)

   GetPrivateProfileStringA(ASection, 0, '', @Buffer, 16383, ::FileName)

   asize(AKeys, 0)
   while( asc(Buffer) <> 0 .and. (n := at(chr(0), Buffer)) > 0 )
      aadd(AKeys, left(Buffer, n - 1))
      Buffer := substr(Buffer, n + 1)
   end
return Self

method TIniFile:ReadSectionValues(ASection, AValues)
   local n, Buffer

   Buffer := space(16383)

   GetPrivateProfileSectionA(ASection, @Buffer, 16383, ::FileName)

   asize(AValues, 0)
   while( asc(Buffer) <> 0 .and. (n := at(chr(0), Buffer)) > 0 )
      aadd(AValues, left(Buffer, n - 1))
      Buffer := substr(Buffer, n + 1)
   end
return Self

method TIniFile:ReadString(ASection, AKey, ADefault)
   local Buffer

   Buffer := space(256)

   GetPrivateProfileStringA(ASection, AKey, ADefault, @Buffer, 256, ::FileName)
return substr(Buffer,1,len(trim(buffer))-1)  // drop the trailing zero
// return trim(Buffer)

method TIniFile:ReadInteger(ASection, AKey, ADefault)
return GetPrivateProfileIntA(ASection, AKey, ADefault, ::FileName)

method TIniFile:ReadBool(ASection, AKey, ADefault)
return (GetPrivateProfileIntA(ASection, AKey, ADefault, ::FileName) <> 0)

method TIniFile:WriteString(ASection, AKey, AString)
return (WritePrivateProfileStringA(ASection, AKey, AString, ::FileName) <> 0)

method TIniFile:WriteInteger(ASection, AKey, AnInt)
return (WritePrivateProfileStringA(ASection, AKey, ltrim(str(AnInt)), ::FileName) <> 0)

method TIniFile:WriteBool(ASection, AKey, ABool)
return (WritePrivateProfileStringA(ASection, AKey, iif(ABool, '1', '0'), ::FileName) <> 0)

method TIniFile:DeleteKey(ASection, AKey)
   WritePrivateProfileStringA(ASection, AKey, 0, ::FileName)
return Self

method TIniFile:EraseSection(ASection)
   WritePrivateProfileStringA(ASection, 0, 0, ::FileName)
return Self

dllfunction GetPrivateProfileSectionNamesA(@ABuffer, ALength, AFileName) using stdcall from "kernel32.dll"
dllfunction GetPrivateProfileSectionA(ASection, @ABuffer, ALength, AFileName) using stdcall from "kernel32.dll"
dllfunction GetPrivateProfileStringA(ASection, AKey, ADefault, @ABuffer, ALength, AFileName) using stdcall from "kernel32.dll"
dllfunction GetPrivateProfileIntA(ASection, AKey, ADefault, AFileName) using stdcall from "kernel32.dll"
dllfunction WritePrivateProfileStringA(ASection, AKey, AString, AFileName) using stdcall from "kernel32.dll"

